﻿Imports PageLayoutControl.PrinterLayerDrawingUtilities
Imports ThinkGeo.MapSuite.Drawing
Imports ThinkGeo.MapSuite.Layers
Imports ThinkGeo.MapSuite.Shapes
Imports ThinkGeo.MapSuite.Styles
Imports ThinkGeo.MapSuite.WinForms

Public Class CustomPrinterTrackOverlay
  Inherits TrackInteractiveOverlay
  Public Property AreaSymbol As AreaStyle
  Public Property LineSymbol As LineStyle
  Public Property PageLayoutParent As SilvicsPageLayout
  Public Property DrawingType As EnumPrinterLayerDrawingType = EnumPrinterLayerDrawingType.None

  Private mbTrackingCancelled As Boolean = False

  Public Sub New()
    AreaSymbol = AreaStyles.CreateSimpleAreaStyle(GeoColors.Transparent, GeoColors.Red, 2)
    TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaSymbol
    LineSymbol = LineStyles.CreateSimpleLineStyle(GeoColors.Red, 2, True)
    TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineSymbol
  End Sub

  Public Enum EnumPagePosition
    TopLeft = 1
    TopRight = 2
    LowerLeft = 3
    LowerRight = 4
    Current = 5
    UseAnchor = 6
    UseClickPoint = 7
  End Enum
  Protected Overrides Sub OnVertexAdding(e As VertexAddingTrackInteractiveOverlayEventArgs)
    If DrawingType = EnumPrinterLayerDrawingType.Arrow Then
      If Me.Vertices.Count = 2 Then
        e.Cancel = True
      End If
    End If
    MyBase.OnVertexAdding(e)
  End Sub

  'Public Property ClickedPoint As PointShape
  Protected Overrides Sub OnTrackEnded(e As TrackEndedTrackInteractiveOverlayEventArgs)
    MyBase.OnTrackEnded(e)
    If Not mbTrackingCancelled Then
      AddDrawShapeElement(e.TrackShape, DrawingType)
    End If
    PageLayoutParent.Refresh()
  End Sub
  Public Sub AddDrawShapeElement(oShape As BaseShape, eDrawingType As EnumPrinterLayerDrawingType)
    Dim dWidthInches As Double
    Dim dHeightInches As Double
    Dim oWorldExtent As RectangleShape = oShape.GetBoundingBox
    Dim oClickedPoint As PointShape
    Dim oDrawingStyle As Style

    If eDrawingType = EnumPrinterLayerDrawingType.Arrow Then
      Dim oLine As LineShape = CType(oShape, LineShape)
      Dim oPoint1 As PointShape = New PointShape(oLine.Vertices(0))
      Dim oPoint2 As PointShape = New PointShape(oLine.Vertices(1))

      oWorldExtent = CreateExtentFromEndPoints(oPoint1, oPoint2)
      Dim eOrientation As EnumPageLayoutArrowDirection = mGetOrientation(oPoint1, oPoint2, oWorldExtent)

      dWidthInches = Math.Abs(PrinterHelper.ConvertLength(oPoint1.X - oPoint2.X, PrintingUnit.Point, PrintingUnit.Inch))
      dHeightInches = Math.Abs(PrinterHelper.ConvertLength(oPoint1.Y - oPoint2.Y, PrintingUnit.Point, PrintingUnit.Inch))

      oClickedPoint = GetClickPoint(oPoint1, oPoint2)

      oDrawingStyle = New CustomPrinterLayerArrowStyle(GeoColors.Black, 2, 1) With {.ArrowOrientation = eOrientation}

    Else
      '***** Get the width and height of the extent of the element being added
      dWidthInches = Math.Abs(PrinterHelper.ConvertLength(oWorldExtent.Width, PrintingUnit.Point, PrintingUnit.Inch))
      dHeightInches = Math.Abs(PrinterHelper.ConvertLength(oWorldExtent.Height, PrintingUnit.Point, PrintingUnit.Inch))

      oClickedPoint = GetClickPoint(oWorldExtent)
      oDrawingStyle = New CustomPrinterLayerShapeStyle(oShape, eDrawingType)
    End If

    Dim oShapePrinterLayer As PrinterLayer
    Dim eResizeMode As PrinterResizeMode
    '***** Set some default properties
    Select Case eDrawingType
      Case EnumPrinterLayerDrawingType.Line,
           EnumPrinterLayerDrawingType.Polygon,
           EnumPrinterLayerDrawingType.Square,
           EnumPrinterLayerDrawingType.Circle,
           EnumPrinterLayerDrawingType.Triangle,
           EnumPrinterLayerDrawingType.Diamond
        eResizeMode = PrinterResizeMode.MaintainAspectRatio
      Case EnumPrinterLayerDrawingType.Ellipse,
           EnumPrinterLayerDrawingType.Rectangle,
           EnumPrinterLayerDrawingType.Arrow
        eResizeMode = PrinterResizeMode.Resizable
    End Select

    If eDrawingType = EnumPrinterLayerDrawingType.Arrow Then
      oShapePrinterLayer = New CustomArrowPrinterLayer()
    Else
      oShapePrinterLayer = New CustomDrawingPrinterLayer()
    End If

    With oShapePrinterLayer
      .DragMode = PrinterDragMode.Draggable
      .SelectabilityMode = SelectabilityMode.NonSelectable
      .ResizeMode = eResizeMode
      .BackgroundMask = CType(oDrawingStyle, AreaStyle)
    End With

    '***** Place the element such that the upper left point of it's extent 
    '***** lies where the user clicked the mouse
    mSetPosition(dWidthInches, dHeightInches, oShapePrinterLayer, EnumPagePosition.UseClickPoint, oClickedPoint)
    PageLayoutParent.PrinterOverlay.PrinterLayers.Add(Guid.NewGuid.ToString, oShapePrinterLayer)
    Me.TrackShapeLayer.InternalFeatures.Clear()

  End Sub

  Private Function mGetOrientation(oPoint1 As PointShape, oPoint2 As PointShape, ByRef oWorldExtent As RectangleShape) As EnumPageLayoutArrowDirection
    Dim eOrientation As EnumPageLayoutArrowDirection
    Dim X1 As Double = oPoint1.X
    Dim Y1 As Double = oPoint1.Y
    Dim X2 As Double = oPoint2.X
    Dim Y2 As Double = oPoint2.Y

    If X1 < X2 Then
      '***** Arrow pointing from left to right, in general
      If Y1 > Y2 Then
        '***** Moving from top to bottom
        eOrientation = EnumPageLayoutArrowDirection.LowerRight
      Else
        eOrientation = EnumPageLayoutArrowDirection.UpperRight
      End If
    Else
      '***** Arrow pointing from right to left, in general
      If Y1 < Y2 Then
        '***** Moving from top to bottom
        eOrientation = EnumPageLayoutArrowDirection.UpperLeft
      Else
        eOrientation = EnumPageLayoutArrowDirection.LowerLeft
      End If
    End If

    '***** If the extent formed by the beginning and end points of the arrow is
    '***** vary narrow it means the arrow is almost completely vertical or
    '***** horizontal. If that is the case we need to doctor up the extent a little
    '***** and set the orientation accordingly.
    If oWorldExtent.Width < 1.5 Then
      '***** Make the width 20
      Select Case eOrientation
        Case EnumPageLayoutArrowDirection.UpperLeft, EnumPageLayoutArrowDirection.UpperRight
          eOrientation = EnumPageLayoutArrowDirection.Up
        Case EnumPageLayoutArrowDirection.LowerLeft, EnumPageLayoutArrowDirection.LowerRight
          eOrientation = EnumPageLayoutArrowDirection.Down
      End Select
      oWorldExtent = mBuildExtentForHorVertDirection(oPoint1, oPoint2, eOrientation)
    ElseIf oWorldExtent.Height < 1.5 Then
      '***** Make the heighth 20
      Select Case eOrientation
        Case EnumPageLayoutArrowDirection.UpperLeft, EnumPageLayoutArrowDirection.LowerLeft
          eOrientation = EnumPageLayoutArrowDirection.Left
        Case EnumPageLayoutArrowDirection.UpperRight, EnumPageLayoutArrowDirection.LowerRight
          eOrientation = EnumPageLayoutArrowDirection.Right
      End Select
      oWorldExtent = mBuildExtentForHorVertDirection(oPoint1, oPoint2, eOrientation)
    End If

    Return eOrientation
  End Function
  Private Function mBuildExtentForHorVertDirection(oPoint1 As PointShape, oPoint2 As PointShape, eOrientation As EnumPageLayoutArrowDirection) As RectangleShape
    Dim oLowerLeftPoint As New PointShape
    Dim oUpperRightPoint As New PointShape
    Dim dDeltaY As Double
    Dim dYFactor As Double
    Dim dDeltaX As Double
    Dim dXFactor As Double

    Select Case eOrientation
      Case EnumPageLayoutArrowDirection.Left, EnumPageLayoutArrowDirection.Right
        '***** Increase the height of the extent that would ordinarily be created by these points to 20
        dDeltaY = Math.Abs(oPoint1.Y - oPoint2.Y)
        dYFactor = 20 - dDeltaY
        '***** The X coordinates will remain unchanged, the Y coordinate will stay the same for 
        '***** oPoint1 and be adjusted for oPoint2
        oPoint2.Y = oPoint1.Y + 10
        oPoint1.Y = oPoint1.Y - 10
      Case EnumPageLayoutArrowDirection.Up, EnumPageLayoutArrowDirection.Down
        '***** Increase the height of the extent that would ordinarily be created by these points to 20
        dDeltaX = Math.Abs(oPoint1.X - oPoint2.X)
        dXFactor = 20 - dDeltaX
        '***** The X coordinates will remain unchanged, the Y coordinate will stay the same for 
        '***** oPoint1 and be adjusted for oPoint2
        oPoint2.X = oPoint1.X + 10
        oPoint1.X = oPoint1.X - 10
    End Select
    Return CreateExtentFromEndPoints(oPoint1, oPoint2)
  End Function
  Private Sub mSetPosition(dWidth As Double, dHeight As Double, oPrinterLayer As PrinterLayer,
                           eInitialPosition As EnumPagePosition, Optional oClickPoint As PointShape = Nothing)


    Dim dCenterPointX As Double
    Dim dCenterPointY As Double
    Dim dDistanceFromEdge As Double = 0.25

    Dim oPageLayoutMap As MapPrinterLayer = PageLayoutParent.PageLayoutMap
    If oPageLayoutMap Is Nothing Then
      Return
    End If
    'Dim oPrinterPage As CustomPagePrinterLayer = mGetPrinterPage()

    '***** Get the bounding box for the MapPrinterLayer
    'oPageLayoutMap.Open()
    'oPrinterPage.Open()
    oPageLayoutMap.Open()
    Dim oPageBoundingBox As RectangleShape = oPageLayoutMap.GetPosition(PrintingUnit.Inch)
    Select Case eInitialPosition
      Case EnumPagePosition.LowerRight
        dCenterPointX = oPageBoundingBox.LowerRightPoint.X - (dWidth / 2) - dDistanceFromEdge
        dCenterPointY = oPageBoundingBox.LowerRightPoint.Y + (dHeight / 2) + dDistanceFromEdge
      Case EnumPagePosition.LowerLeft
        dCenterPointX = oPageBoundingBox.LowerLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
        dCenterPointY = oPageBoundingBox.LowerLeftPoint.Y + (dHeight / 2) + dDistanceFromEdge
      Case EnumPagePosition.TopRight
        dCenterPointX = oPageBoundingBox.UpperRightPoint.X - (dWidth / 2) - dDistanceFromEdge
        dCenterPointY = oPageBoundingBox.UpperRightPoint.Y - (dHeight / 2) - dDistanceFromEdge
      Case EnumPagePosition.TopLeft
        dCenterPointX = oPageBoundingBox.UpperLeftPoint.X + (dWidth / 2) + dDistanceFromEdge
        dCenterPointY = oPageBoundingBox.UpperLeftPoint.Y - (dHeight / 2) - dDistanceFromEdge
      Case EnumPagePosition.Current
        dCenterPointX = oPrinterLayer.GetPosition(PrintingUnit.Inch).GetCenterPoint.X
        dCenterPointY = oPrinterLayer.GetPosition(PrintingUnit.Inch).GetCenterPoint.Y
      Case EnumPagePosition.UseClickPoint
        dCenterPointX = oClickPoint.X + dWidth / 2
        dCenterPointY = oClickPoint.Y - dHeight / 2
    End Select
    '***** Now let's try to place this layer in the lower right corner of the page
    oPrinterLayer.SetPosition(dWidth, dHeight, dCenterPointX, dCenterPointY, PrintingUnit.Inch)

  End Sub
  Protected Overrides Sub DrawCore(oGeoCanvas As GeoCanvas)
    Dim oTrianglePoints As New List(Of ScreenPointF())
    Dim oExtent As RectangleShape ' = Me.GetTrackingShape.GetBoundingBox
    Dim oTrackShape = Me.GetTrackingShape

    If oTrackShape Is Nothing Then
      Return
    End If

    oExtent = oTrackShape.GetBoundingBox
    If oExtent.Width < 1 Or oExtent.Height < 1 Then
      Return
    End If

    If Me.TrackMode = TrackMode.Rectangle Then
      Select Case DrawingType
        Case EnumPrinterLayerDrawingType.Diamond
          Dim oDiamondPoints As New List(Of ScreenPointF())
          Dim oScreenPoints(4) As ScreenPointF
          Dim oScreenExtent = ExtentHelper.ToScreenCoordinate(oGeoCanvas.CurrentWorldExtent, oExtent, oGeoCanvas.Width, oGeoCanvas.Height)
          With oScreenExtent
            oScreenPoints(0) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MaximumY) '<--- Bottom center
            oScreenPoints(1) = New ScreenPointF(.MaximumX, .MaximumY - CSng(.Height / 2)) '<--- Right side
            oScreenPoints(2) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MinimumY) '<--- Center top
            oScreenPoints(3) = New ScreenPointF(.MinimumX, .MaximumY - CSng(.Height / 2)) '<--- Left side
            oScreenPoints(4) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MaximumY) '<--- Bottom center
            oDiamondPoints.Add(oScreenPoints)
          End With
          oGeoCanvas.DrawArea(oDiamondPoints, New GeoPen(AreaSymbol.OutlinePen.Color, AreaSymbol.OutlinePen.Width), New GeoSolidBrush(AreaSymbol.FillSolidBrush.Color), ThinkGeo.MapSuite.Drawing.DrawingLevel.LabelLevel, 0, 0, ThinkGeo.MapSuite.Drawing.PenBrushDrawingOrder.BrushFirst)
        Case EnumPrinterLayerDrawingType.Triangle
          Dim oScreenExtent = ExtentHelper.ToScreenCoordinate(oGeoCanvas.CurrentWorldExtent, oExtent, oGeoCanvas.Width, oGeoCanvas.Height)
          Dim oScreenPoints(3) As ScreenPointF
          With oScreenExtent
            oScreenPoints(0) = New ScreenPointF(.MinimumX, .MaximumY) '<--- Lower Left
            oScreenPoints(1) = New ScreenPointF(.MaximumX, .MaximumY) '<--- Lower Right
            oScreenPoints(2) = New ScreenPointF(CSng(.MinimumX + .Width / 2), .MinimumY) '<--- Center top
            oScreenPoints(3) = New ScreenPointF(.MinimumX, .MaximumY) '<--- Lower Left
            oTrianglePoints.Add(oScreenPoints)
          End With
          oGeoCanvas.DrawArea(oTrianglePoints, New GeoPen(AreaSymbol.OutlinePen.Color, AreaSymbol.OutlinePen.Width), New GeoSolidBrush(AreaSymbol.FillSolidBrush.Color), ThinkGeo.MapSuite.Drawing.DrawingLevel.LabelLevel, 0, 0, ThinkGeo.MapSuite.Drawing.PenBrushDrawingOrder.BrushFirst)
        Case Else
          MyBase.DrawCore(oGeoCanvas)
      End Select
    Else
      MyBase.DrawCore(oGeoCanvas)
    End If
  End Sub
  Protected Overrides Function GetTrackingShapeCore() As BaseShape
    Dim oShape As BaseShape = MyBase.GetTrackingShapeCore()
    '***** This is to prevent the square shape from rotating as it is being
    '***** drawn by the user.
    '***** SET 04/02/2019
    If Me.TrackMode = TrackMode.Square AndAlso oShape IsNot Nothing Then
      Dim oRectangle As RectangleShape = oShape.GetBoundingBox()
      oRectangle = New RectangleShape(oRectangle.UpperLeftPoint, New PointShape(oRectangle.UpperLeftPoint.X + oRectangle.Width, oRectangle.UpperLeftPoint.Y - oRectangle.Height))
      oShape = oRectangle
    End If

    Return oShape
  End Function
  Public Sub TrackOverlayCancelled()
    mbTrackingCancelled = True
    If TrackMode = TrackMode.Polygon OrElse TrackMode = TrackMode.Line Then
      EndTracking()
      MouseDoubleClick(New InteractionArguments())
    Else
      MouseUp(New InteractionArguments())
    End If
    mbTrackingCancelled = False
  End Sub

  'Private Sub CustomPrinterTrackOverlay_MapMouseClick(sender As Object, e As MapMouseClickInteractiveOverlayEventArgs) Handles Me.MapMouseClick

  '  InMemDisplayLayer.InternalFeatures.Clear()

  '  Dim oScreenPoint As New ScreenPointF(e.InteractionArguments.ScreenX, e.InteractionArguments.ScreenY)
  '  Dim oMapPoint As PointShape = PrinterHelper.ConvertScreenCoordinateToMapPrinterCoordinate(
  '                                                                      oScreenPoint,
  '                                                                      PageLayoutParent.PageLayoutMap, '<-- MapPrinterLayer
  '                                                                      e.InteractionArguments.CurrentExtent,
  '                                                                      e.InteractionArguments.MapWidth,
  '                                                                      e.InteractionArguments.MapHeight)

  '  Debug.WriteLine("X = " + oMapPoint.X.ToString + ", Y = " + oMapPoint.Y.ToString)

  '  InMemDisplayLayer.InternalFeatures.Add(oMapPoint.GetFeature)
  '  InMemDisplayLayer.BuildIndex()
  '  PageLayoutParent.PageLayoutMap.IsDrawing = True
  '  PageLayoutParent.Refresh()
  '  PageLayoutParent.PageLayoutMap.IsDrawing = False
  'End Sub
End Class
